var BASE_URL = 'http://dict.tu-chemnitz.de';

var HEIGHT_OF_STATIC_WINDOW_COMPONENTS_NO_PAGESWITCHER = 75;
var HEIGHT_OF_STATIC_WINDOW_COMPONENTS_WITH_PAGESWITCHER = 85;
var currentHeightOfStaticWindowComponents = 75;

// urls for navigation buttons, will be set by #showNavigation()
var backLinkUrl;
var nextLinkUrl;

// getLocalizedString() pulls a string out an array named localizedStrings.  Each language project
// directory in this widget contains a file named "localizedStrings.js", which, in turn, contains
// an array called localizedStrings.  This method queries the array of the file of whichever language
// has highest precidence, according to the International pane of System Preferences.
function getLocalizedString(key)
{
    try {
        var ret = localizedStrings[key];
        if (ret === undefined)
            ret = key;
        return ret;
    } catch (ex) {}
    return key;
}


function setup()
{
    // getting localization strings to the html elements
    document.getElementById('searchBt').value = getLocalizedString('Search');
    document.getElementById('service').title = getLocalizedString('Choose the searchmethod');
    document.getElementById('deen').text = getLocalizedString('DeEnDe Dict');
    document.getElementById('de-en').text = getLocalizedString('De-En Dict');
    document.getElementById('en-de').text = getLocalizedString('En-De Dict');
    document.getElementById('de-en-ex').text = getLocalizedString('De-En Examples');
    document.getElementById('dict-en').text = getLocalizedString('Definitions En');
    document.getElementById('dict-de').text = getLocalizedString('Synonyms De');
    document.getElementById('fortune-en').text = getLocalizedString('Sayings En');
    document.getElementById('fortune-de').text = getLocalizedString('Sayings De');
    document.getElementById('dees').text = getLocalizedString('DeEsDe Dict');
    document.getElementById('de-es').text = getLocalizedString('De-Es Dict');
    document.getElementById('es-de').text = getLocalizedString('Es-De Dict');
    document.getElementById('fortune-es').text = getLocalizedString('Sayings Es');
    document.getElementById('de-es-ex').text = getLocalizedString('De-Es Examples');
}

/*
 * query the BeoLingus site via ajax-get.
 * Uses the data the user gave into the formular.
 * Invoked via the search-button (or return).
 */
function loadResult()
{
    var query = document.getElementById('query').value;
    var service = getService();

    $.get(BASE_URL + "/dings.cgi", 
        { mini: "1", count: "5", lang: getLocalizedString('Language'), query: query, service: service }, 
        function(data) {
            showQueryResult(data);
            /*
             * we have to use a small timeout because we already use a timeout
             * in showQueryResult().
             * Without this timeout the fadeIn of the widget would look awful
             */
            window.setTimeout("showResultWindow()", 45);
    });
    
    return false;
}

/*
 * The links on the loaded page should be shown also in the widget.
 * Since this works differently than the search through our formular (there is a url) we handle this here.
 * The links are tranformed before to point to this method (@see setBackLinks()).
 */
function followLink(url)
{
    /*
     * this is a link to the speech feature.
     * We have to handle it different to links to other words
     */
    if(url.indexOf('?speak=') != -1)
    {
        playSpeech(url);
    }
    else
    {
        $.get(BASE_URL + url, null, 
            function(data) {
                showQueryResult(data);
        });
    }
}

/*
 * shows the result of the ajax-request in the widget
 *
 * @param data the response of the ajax-request, a html document
 */
function showQueryResult(data)
{
    var startResult = data.indexOf('<table id="result"');
    var endResult = data.lastIndexOf('</tbody>');
    var result = data.substring(startResult, endResult + 8);
    result = result + "</table>";
    
    $("#miniResultDiv").html(result);
    setSrc("img", "src");
    setBackLinks();
    showNavigation(data);
    
    /*
     * we have to wait a little bit with resizing
     * so that the images can be loaded.
     * The resizing depends on the actual height of the html,
     * and without loaded images this would be too small,
     * not all text would be shown
     */
    window.setTimeout("resizeResultDiv()", 50);
}

/*
 * returns the selected service (de <> en, de > es, ...)
 */
function getService()
{
    var service = document.getElementById('service');

    for (i = 0; i < service.length; ++i)
    {
        if (service.options[i].selected == true)
        {
            return service.options[i].value;
        }
    }
}

/*
 * changes the src-attribute of all images so that they point to the original website
 */
function setSrc(element, attribute)
{
    $("#miniResultDiv").find(element).each(function(i) {
        var newSrc = BASE_URL + $(this).attr(attribute);
        $(this).attr(attribute, newSrc);
    });
}

/*
 * Changes all links so that they show to a javascript function
 */
function setBackLinks()
{
    $("#miniResultDiv").find("a").each(function(i) {
        var newSrc = "javascript:followLink('" + $(this).attr("href") + "');";
        $(this).attr("href", newSrc);
    });
}

function showResultWindow()
{
    $("#resultDiv").fadeIn("slow");
}

function resizeResultDiv()
{
    var resultDivHeight = $("#result").height() + HEIGHT_OF_STATIC_WINDOW_COMPONENTS_NO_PAGESWITCHER;
    $("#resultDiv").height(resultDivHeight);
    var resultWindowHeight = $("#result").height() + currentHeightOfStaticWindowComponents;
    window.resizeTo(300, resultWindowHeight);
}

/*
 * shows navigation buttons if there are more than the default 
 * results (for and maybe back)
 *
 * @param resultHtml the html that came from the BeoLingus Website
 */
function showNavigation(resultHtml)
{
    /*
     * find out if there are links to other result-pages in the html
     * we got from the ajax-request
     */
    var hasBackLink = resultHtml.indexOf('<tr class="head"><td class="tip" colspan="3" align="center">\n<br /><strong><a') != -1;
    var hasNextLink = resultHtml.indexOf('<tr class="head"><td class="tip"') != -1 
        && resultHtml.indexOf('&gt;&gt;&gt;&gt;&gt;</strong><br />\n<br /></td></tr></table>') != -1
    
    /*
     * extract the links for the navigation buttons
     */
    if(hasBackLink)
    {
        var start = resultHtml.indexOf('<tr class="head"><td class="tip" colspan="3" align="center">\n<br /><strong><a href="');
        var end = resultHtml.indexOf('">&lt;', start);
        
        lastLinkUrl = resultHtml.substring(start + 84, end);
    }
    
    if(hasNextLink)
    {
        var preStart = resultHtml.indexOf('<tr class="head"><td class="tip" colspan="3" align="center">\n<br /><strong>') + 81;
        var start = resultHtml.indexOf(' <a href="', preStart);
        var end = resultHtml.indexOf('">&gt;', start);
        
        nextLinkUrl = resultHtml.substring(start + 10, end);
    }
    
    /*
     * show or hide the navigation buttons
     */  
    // possibility one: there are more results, we are on page one
    if( hasNextLink && !hasBackLink )
    {
        showNavigationDiv();
        $('#lastBt').css('display', 'none');
        $('#nextBt').css('display', 'inline');
    }
    // poss two: there are more and we are already on page two
    else if( hasBackLink && hasNextLink )
    {
        showNavigationDiv();
        $('#lastBt').css('display', 'inline');
        $('#nextBt').css('display', 'inline');
    }
    // poss three: this is the last page, we only can flip back
    else if( hasBackLink && !hasNextLink)
    {
        showNavigationDiv();
        $('#lastBt').css('display', 'inline');
        $('#nextBt').css('display', 'none');
    }
    // poss four: there are no pages, results fit on one page
    else
    {
        currentHeightOfStaticWindowComponents = HEIGHT_OF_STATIC_WINDOW_COMPONENTS_NO_PAGESWITCHER;
        $('#pageNavigation').css('display', 'none');
    }
}

function showNavigationDiv()
{
    currentHeightOfStaticWindowComponents = HEIGHT_OF_STATIC_WINDOW_COMPONENTS_WITH_PAGESWITCHER;
    $('#pageNavigation').css('display', 'inline');
}

function loadLastPage()
{
    followLink(lastLinkUrl);
}

function laodNextPage()
{
    followLink(nextLinkUrl);
}

/*
 * In BeoLingus there's the cool feature that some text can be
 * heard. This is not a synthi-voice, but actually recorded.
 * Normally there a window with a flash player will be opened, but there's also a link to 
 * an mp3-file.
 *
 * This function loads the page with the flash player, extracts the mp3 link, and plays the mp3 
 * in the background.
 */
function playSpeech(url)
{
    $.get("http://dict.tu-chemnitz.de" + url, null, 
            function(data) {
                playMp3InPage(data);
        });
}

/*
 * @see playSpeech(url)
 */
function playMp3InPage(html)
{
    var start = html.indexOf('<a href="');
    var end = html.indexOf('.mp3">');
    var link = html.substring(start + 9, end + 4);
    
    document.getElementById("speachMp3").SetURL(BASE_URL + link);
}
